<#ftl strip_whitespace=true>
<#--
 * insight.ftl
 *
 * This file consists of a collection of FreeMarker macros aimed at easing
 * some of the common requirements for Insight Operation rendering.
 *
 * @version 1.0
 -->

<#--
 * title
 *
 * Macro to print the body as a title.
 -->
<#macro title>
    <h2><#nested></h2>
</#macro>

<#--
 * print
 *
 * Macro to print the item html encoded.  Due to limitations in Freemarker, 
 * null and empty string are handled identically.  If null detection is 
 * important, it must be handled before invoking this macro.
 *
 * @param item : value to print
 -->
<#macro print item="">
    <#if !item?has_content>
        <em>empty</em>
    <#elseif item?is_enumerable>
    	<em>array[${item?size?html}]</em>
    <#elseif item?is_date>
    	${item?datetime?string.medium?html}
    <#elseif item?is_number>
        ${item?html}
    <#else>
        ${item?string?html}
    </#if>
</#macro>

<#--
 * group
 *
 * Render a group of name/value pairs.
 *
 * @param [label] : label for the group
 * @param [collection] : items to iterate on
 * @param [if] : if false, the group is not rendered
 -->
<#macro group label="" collection="" if="">
    <@test test=if>
        <#if label?has_content>
            <@title>${label?html}</@title>
        </#if>
        <table class="dl">
            <#if collection?is_enumerable>
                <#list collection as c>
                    <#nested c />
                </#list>
            <#else>
                <#nested />
            </#if>
        </table>
    </@test>
</#macro>

<#--
 * entry
 *
 * Render a name/value pair member of a group.  Must be nested under the group macro.
 *
 * By default, the entry is skipped if the value is empty.
 *
 * @param name : the name portion of the name/value pair
 * @param [value] : the value portion of the name/value pair
 * @param [required] : if true, the entry is rendered even if the value is empty
 * @param [if] : if false, the entry is not rendered
 -->
<#macro entry name value="" required="" if="">
    <@test test=if>
        <#local body><#nested /></#local>
        <#if required?has_content || body?has_content || value?has_content>
            <tr>
                <td>${name?html}</td>
                <td>
                    <#if body?has_content>
                        ${body}
                    <#else>
                        <@print item=value />
                    </#if>
                </td>
            </tr>
        </#if>
    </@test>
</#macro>

<#--
 * list
 *
 * Render a group of values.
 *
 * @param [type] : 'ordered' or 'unordered'.  Unordered by default
 * @param [label] : label for the list
 * @param [collection] : items to iterate on
 * @param [if] : if false, the entry is not rendered
 -->
<#macro list type="" label="" collection="" if="">
    <@test test=if>
        <#if label?has_content>
            <@title>${label?html}</@title>
        </#if>
        <#if type == "ordered">
            <#local listType = "ol" />
        <#else>
            <#local listType = "ul" />
        </#if>
        <${listType}>
        <#if collection?is_enumerable>
            <#list collection as c>
                <#local body><#nested c /></#local>
                <#if body?has_content>
                    ${body}
                <#else>
                    <@listEntry value=c />
                </#if>
            </#list>
        <#else>
        	<#nested />
        </#if>
        </${listType}>
    </@test>
</#macro>

<#--
 * listEntry
 *
 * Render a member of a list.  Must be nested under the list macro.
 *
 * By default, the entry is skipped if the value is empty.
 *
 * @param [value] : the list member
 * @param [required] : if true, the entry is rendered even if the value is empty
 * @param [if] : if false, the entry is not rendered
 -->
<#macro listEntry value="" required="" if="">
    <@test test=if>
        <#local body><#nested /></#local>
        <#if required?has_content || body?has_content || value?has_content>
            <li>
                <#if body?has_content>
                    ${body}
                <#else>
                    <@print item=value />
                </#if>
            </li>
        </#if>
    </@test>
</#macro>

<#--
 * sourceCodeLocation
 *
 * Render location in source code.  The class and method names are rendered by default.
 *
 * @param location : the SourceCodeLocation object
 * @param [class] : 'show' or 'hide' the class name.  Default is 'show'
 * @param [method] : 'show' or 'hide' the method name.  Default is 'show'
 * @param [line] : 'show' or 'hide' the line number.  Default is 'hide'
 -->
<#macro sourceCodeLocation location class="show" method="show" line="hide">
    <@test test=location?has_content>
        <@insight.group label="Source Code Location">
	        <@entry name="Class Name" value=location.className if=class == 'show' />
	        <@entry name="Method Name" value=location.methodName if=method == 'show' />
	        <@entry name="Line Number" value=location.lineNumber if=line == 'show' />
        </@insight.group>
    </@test>
</#macro>

<#--
* test
*
* private, internal use
-->
<#macro test test="">
    <#if !test?is_boolean || test>
        <#nested />
    </#if>
</#macro>
